<?php

namespace Drupal\current_date_time\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\current_date_time\Services\TimestampService;
use Symfony\Component\DependencyInjection\ContainerInterface;
/**
 * Provides a custom date and time block.
 *
 * @Block(
 *   id = "user_block",
 *   admin_label = @Translation("Current-time(User based)"),
 *   category = @Translation("Custom")
 * )
 */
class UserBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The timestamp service.
   *
   * @var \Drupal\current_date_time\Services\TimestampService
   */
  protected $timestampService;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs a new TimeBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\current_date_time\Services\TimestampService $timestamp_service
   *   The timestamp service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TimestampService $timestamp_service, AccountInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->timestampService = $timestamp_service;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get("current_date_time.timestamp_service"),
      $container->get("current_user")
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $date_time_format = $config['date_time_format'] ?? 'd/m/Y [g:i:s A]';
    $current_timestamp = REQUEST_TIME;
    // Get the current user's timezone.
    $user_timezone = $this->currentUser->getTimezone();

    // Use the custom timestamp service.
    $formatted_timestamp = $this->timestampService->getFormattedTimestamp($current_timestamp, $date_time_format, $user_timezone);
    $build = [
      '#markup' => '<div id="custom-time-block">' . $this->t(
          'Date and Time: @datetime', [
            '@datetime' => $formatted_timestamp,
          ]
      ) . '</div>',
      '#cache' => [
        'context' => ['user'],
        'tags' => ['user:' . $this->currentUser->id()],
      ],
    ];
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();
    $form['date_time_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Date And Time Format'),
      '#default_value' => $config['date_time_format'] ?? 'd/m/Y [g:i:s A]',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfigurationValue('date_time_format', $form_state->getValue('date_time_format'));
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    // Set a reasonable cache max age, e.g., 1 hour (3600 seconds).
    return 10;
  }

}
